"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var mn_req_nodata_component_1 = require("./mn-req-nodata.component");
var mn_req_service_1 = require("./mn-req.service");
var http_1 = require("@angular/common/http");
var mu = require("mzmu");
var MnReqHttpComponent = (function () {
    function MnReqHttpComponent(_http, _rs) {
        var _this = this;
        this._http = _http;
        this._rs = _rs;
        this.result = new core_1.EventEmitter();
        this._showGutter = true;
        this._showLoading = true;
        this._noDataComponent = this._rs._noDataComponent || mn_req_nodata_component_1.MnReqNoDataComponent;
        this._showNoData = true;
        this._restful = true;
        this._isNoData = false;
        this._process = 0;
        this._delay = 500;
        this.debounceReqHttp = mu.debounce(function (req) {
            mu.run(req, function () {
                _this.ngOnDestroy();
                _this.reqHttp(req);
            });
        }, this._delay);
        // 处理直接传递数据
        this.debounceData = mu.debounce(function (change) {
            // data 只认存在数据（isExist)
            var _data = mu.prop(change, 'currentValue.data') || mu.prop(change, 'currentValue');
            if (mu.isExist(_data)) {
                _this._dbTimeId && clearTimeout(_this._dbTimeId);
                _this._process = 99;
                _this._dbTimeId = setTimeout(function () {
                    _this._isNoData = mu.isEmpty(_data);
                    _this.result.emit(_data);
                    _this._process = 100;
                }, 2500);
            }
        }, this._delay);
        // 获取 resource pool
        this._restful = mu.ifnvl(this._rs._restful, true);
    }
    MnReqHttpComponent.prototype.reqHttp = function (req) {
        var _this = this;
        var args = [];
        var method = req.method || (req.payload ? 'post' : 'get');
        switch (method) {
            case 'get':
            case 'delete':
                args = [req.params];
                break;
            case 'patch':
            case 'post':
                args = [
                    req.payload,
                    req.params
                ];
                break;
        }
        var source = mu.run(req.url, function (url) {
            return (_a = _this._http)[method].apply(_a, [url].concat(args));
            var _a;
        }, function () {
            var _resources = _this._rs.getResources();
            return _resources[req.api][method](req.params || {}, req.payload || {});
        });
        // this._observable && this._observable.unsubscribe();
        this._observable = source.subscribe(function (res) {
            _this._process = 100;
            res = res || {};
            mu.run(_this._restful ? res.data : res, function () {
                _this._isNoData = false;
            }, function () {
                _this._isNoData = true;
            });
            _this.result.emit(res);
        }, 
        // error
        // error
        function () {
            _this._process = 100;
            _this._isNoData = true;
            _this.result.emit({});
        }, 
        // completed
        // completed
        function () {
            _this._process = 100;
        });
    };
    MnReqHttpComponent.prototype.ngOnChanges = function (changes) {
        var _this = this;
        mu.run(this.req, function () {
            _this._process = mu.randomInt(0, 49);
            _this.processStep();
        });
        mu.run(changes['req'], function () {
            _this.debounceReqHttp(_this.req);
        });
        /**
                 * req 与 data 二者只可同时生效其一
                 * req 占主导位置
                 */
        mu.run(!this.req && changes['_data'], function () {
            _this._process = mu.randomInt(_this._process || 30, 60);
            _this.debounceData(changes['_data']);
        });
    };
    MnReqHttpComponent.prototype.ngOnDestroy = function () {
        this._observable && this._observable.unsubscribe();
    };
    MnReqHttpComponent.prototype.processStep = function () {
        var _this = this;
        var tid = setTimeout(function () {
            if (_this._process < mu.randomInt(75, 85)) {
                _this._process = _this._process * 1.05;
                _this.processStep();
            }
            else {
                clearTimeout(tid);
            }
        }, mu.randomInt(300, 1200));
    };
    MnReqHttpComponent.decorators = [
        { type: core_1.Component, args: [{
                    selector: 'mn-req, mn-http',
                    template: "\n        <ng-template [ngIf]=\"_showLoading\">\n            <mn-loader-bar [loader]=\"loader\"\n                           [loaderStyle]=\"loaderStyle\"\n                           [progress]=\"_process\"></mn-loader-bar>\n        </ng-template>\n\n        <ng-template [ngIf]=\"_showGutter\">\n            <mn-gutter [mnRows]=\"1\" *ngIf=\"_process != 100\"></mn-gutter>\n        </ng-template>\n\n        <ng-container *ngIf=\"_showNoData\">\n            <mn-dynamic-component\n                    *ngIf=\"_isNoData && _process === 100\"\n                    [component]=\"_noDataComponent\"\n                    [inputs]=\"_noDataContext\">\n            </mn-dynamic-component>\n        </ng-container>\n\n        <ng-container *ngIf=\"_showNoData ? !_isNoData : true\">\n            <ng-content></ng-content>\n        </ng-container>\n    ",
                    styles: ['mn-req {  position: relative;  display: block;  width: 100%;  height: 100%;}mn-req mn-gutter {  position: absolute;  left: 0;  top: 0;  bottom: 0;  right: 0;  overflow: hidden;  z-index: 2;  padding: 16px;  background: #fff;  animation-name: mnGutterfadeOut;  /*动画名称*/  animation-duration: 5s;  /*动画持续时间*/  animation-iteration-count: 1;  /*动画次数*/  animation-delay: 0s;  /*延迟时间*/  animation-fill-mode: forwards;  /*停在动画最后一帧*/}mn-req mn-gutter mn-row mn-col {  height: 24px;  padding-bottom: 5px;  padding-top: 5px;}mn-req mn-gutter mn-row mn-col .mnc-content {  background: rgba(125, 125, 125, 0.1);  border-radius: 2px;}mn-req mn-gutter.mnc-hide {  opacity: 0.1;}mn-req mn-dynamic-component {  display: block;  animation-name: noDataFadeIn;  /*动画名称*/  animation-duration: 2s;  /*动画持续时间*/  animation-iteration-count: 1;  /*动画次数*/  animation-delay: 0s;  /*延迟时间*/}@-webkit-keyframes mnGutterfadeOut {  0% {    opacity: 1;    /*初始状态 透明度为0*/  }  20% {    opacity: .8;    /*中间状态 透明度为0*/  }  30% {    opacity: .6;    /*中间状态 透明度为0*/  }  60% {    opacity: .5;    /*中间状态 透明度为0*/  }  100% {    opacity: .1;    /*结尾状态 透明度为1*/  }}@-webkit-keyframes noDataFadeIn {  0% {    opacity: 0;    /*初始状态 透明度为0*/  }  50% {    opacity: .5;    /*中间状态 透明度为0*/  }  100% {    opacity: 1;    /*结尾状态 透明度为1*/  }}'],
                    encapsulation: core_1.ViewEncapsulation.None
                },] },
    ];
    /** @nocollapse */
    MnReqHttpComponent.ctorParameters = function () { return [
        { type: http_1.HttpClient, },
        { type: mn_req_service_1.MnReqServices, },
    ]; };
    MnReqHttpComponent.propDecorators = {
        "result": [{ type: core_1.Output },],
        "req": [{ type: core_1.Input },],
        "_data": [{ type: core_1.Input, args: ['mnData',] },],
        "_showGutter": [{ type: core_1.Input, args: ['mnShowGutter',] },],
        "_showLoading": [{ type: core_1.Input, args: ['mnShowLoading',] },],
        "_noDataComponent": [{ type: core_1.Input, args: ['mnNoDataComponent',] },],
        "_noDataContext": [{ type: core_1.Input, args: ['mnNoDataContext',] },],
        "_showNoData": [{ type: core_1.Input, args: ['mnShowNoData',] },],
        "loader": [{ type: core_1.Input },],
        "loaderStyle": [{ type: core_1.Input },],
    };
    return MnReqHttpComponent;
}());
exports.MnReqHttpComponent = MnReqHttpComponent;
//# sourceMappingURL=mn-req.component.js.map